#!/usr/bin/env python3
"""
This example illustrates how to plot an NPStat 1-d density using Matplotlib
"""

__author__="Igor Volobouev (i.volobouev@ttu.edu)"
__version__="1.0"
__date__ ="Sep 13 2019"

# Perform necessary imports
import npstat as ns
import matplotlib.pyplot as plt

# d1 is an object of any class which inherits from AbsDistribution1D
d1 = ns.Gauss1D(0, 1)

# Generate xdata, ydata sequence for making the plot. Apart from
# the distribution, the NPStat "scanDensity1D" function takes the
# the minimum x value (xmin), the maximum x value (xmax), and the
# number of points (n), as arguments. The x coordinates of the points
# will be at xmin + i*step, where step = (xmax - xmin)/(n - 1) and
# i = 0, 1, ..., n - 1.
xmin = -4.0
xmax = 4.0
n = 500
x, y = ns.scanDensity1D(d1, xmin, xmax, n)

# Make a plot by drawing a simple line 
plt.plot(x, y)

# Set the axis labels and the plot title
plt.xlabel('X')
plt.ylabel('Probability density')
plt.title('NPStat Distribution {}'.format(type(d1).__name__))

# Display the window. The code enters Matplotlib event loop here.
# You will be able to use the window interface to manipulate the
# figure (pan, zoom) and to save it in various graphical formats.
plt.show()
