#!/usr/bin/env python3
"""
This example illustrates how to plot an NPStat 2-d density using Matplotlib
"""

__author__="Igor Volobouev (i.volobouev@ttu.edu)"
__version__="1.0"
__date__ ="Sep 6 2022"

# Perform necessary imports
import npstat as ns
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
from npstat_plotting import colormeshDensity2D, surfacePlotDensity2D
from npstat_utils import BoxND

# Distribution to plot
mu = (1.0, 0.0)
covarianceMatrix = ((2.0, 0.5),
                    (0.5, 0.5))
d1 = ns.GaussND(mu, covarianceMatrix)

# Parameters for making the plot (bounding
# box and number of discretization cells)
bBox = BoxND((-3, 5), (-4, 4))
nDiscreteColor = 1000
nDiscreteSurface = 81

# Make the plot
fig = plt.figure(num="2-D Density Plotting Methods")

# Colormap plot on the left
ax0 = fig.add_subplot(121)
mesh = colormeshDensity2D(ax0, d1, bBox,
                          (nDiscreteColor, nDiscreteColor),
                          title="2-D Density Color Mesh")
divider = make_axes_locatable(ax0)
cax = divider.append_axes("right", size="7%", pad=0.05)
plt.colorbar(mesh, cax=cax)

# Surface plot on the right
ax1 = fig.add_subplot(122, projection='3d')
surface = surfacePlotDensity2D(ax1, d1, bBox,
                               (nDiscreteSurface, nDiscreteSurface),
                               title="2-D Density Surface Plot",
                               cmap=matplotlib.cm.coolwarm)
plt.colorbar(surface, shrink=0.7, pad=0.1)

# Finalize plot layout
fig.set(figheight=6, figwidth=13)
fig.tight_layout()

# Display the window. The code enters Matplotlib event loop here.
plt.show()
