#include <stdexcept>

#include "npstat/nm/GaussLegendreQuadrature.hh"
#include "npstat/nm/GaussLegendreQuadrature2D.hh"

namespace npstat {
    GaussLegendreQuadrature2D::GaussLegendreQuadrature2D(
        const unsigned npoints0, const unsigned npoints1)
        : a0_(npoints0/2U), w0_(npoints0/2U),
          a1_(npoints1/2U), w1_(npoints1/2U),
          npoints0_(npoints0), npoints1_(npoints1)
    {
        GaussLegendreQuadrature glq0(npoints0);
        glq0.getAbscissae(&a0_[0], a0_.size());
        glq0.getWeights(&w0_[0], w0_.size());

        GaussLegendreQuadrature glq1(npoints1);
        glq1.getAbscissae(&a1_[0], a1_.size());
        glq1.getWeights(&w1_[0], w1_.size());
    }

    unsigned GaussLegendreQuadrature2D::npoints(const unsigned which) const
    {
        switch (which)
        {
        case 0U:
            return npoints0_;

        case 1U:
            return npoints1_;

        default:
            throw std::invalid_argument(
                "In npstat::GaussLegendreQuadrature2D::npoints: "
                "dimension number is out of range");
            return 0U;
        }
    }
}
