#ifndef NPSTAT_RECURRENCECOEFFS_HH_
#define NPSTAT_RECURRENCECOEFFS_HH_

#include <cassert>
#include <vector>
#include <utility>
#include <cmath>

#include "npstat/nm/OrthoPolyMethod.hh"

namespace npstat {
    namespace Private {
        struct Recurrence
        {
            inline Recurrence(const long double a,
                              const long double b)
                : alpha(a), beta(b)
                {
                    assert(beta > 0.0L);
                    sqrbeta = sqrtl(beta);
                }

            long double alpha;
            long double beta;
            long double sqrbeta;
        };

        // The following function should be the only one
        // directly used by the rest of the package
        template <class MeasurePt>
        void calcRecurrenceCoeffs(const std::vector<MeasurePt>& measure,
                                  unsigned maxdeg, long double norm,
                                  OrthoPolyMethod m,
                                  std::vector<Recurrence>* rcoeffs);

        template <class MeasurePt>
        void calcRecurrenceStieltjes(const std::vector<MeasurePt>& measure,
                                     unsigned maxdeg, long double norm,
                                     std::vector<Recurrence>* rcoeffs);

        template <class MeasurePt>
        void calcRecurrenceLanczos(const std::vector<MeasurePt>& measure,
                                   unsigned maxdeg, long double norm,
                                   std::vector<Recurrence>* rcoeffs);

        template <class MeasurePt>
        std::pair<long double,long double> monicInnerProducts(
            const std::vector<Recurrence>& rcoeffs,
            const std::vector<MeasurePt>& measure,
            unsigned degree, long double norm);

        long double monicpoly(const std::vector<Recurrence>& rcoeffs,
                              unsigned degree, long double x);
    }
}

#include "npstat/nm/RecurrenceCoeffs.icc"

#endif // NPSTAT_RECURRENCECOEFFS_HH_
