#ifndef NPSTAT_SCANEXTREMUM1D_HH_
#define NPSTAT_SCANEXTREMUM1D_HH_

/*!
// \file ScanExtremum1D.hh
//
// \brief Finding extrema of scanned 1-d curves
//
// Author: I. Volobouev
//
// June 2022
*/

namespace npstat {
    class ScanExtremum1D
    {
    public:
        inline ScanExtremum1D(const double loc, const double val,
                              const bool isMin, const bool b)
            : location_(loc), value_(val),
              isMinimum_(isMin), onTheBoundary_(b) {}

        inline double location() const {return location_;}
        inline double value() const {return value_;}
        inline bool isMinimum() const {return isMinimum_;}
        inline bool isOnTheBoundary() const {return onTheBoundary_;}

    private:
        double location_;
        double value_;
        bool isMinimum_;
        bool onTheBoundary_;
    };

    //@{
    /**
    // The input arrays must have at least two elements.
    // Coordinates must be monotonously increasing.
    */
    ScanExtremum1D findScanMinimum1D(const double* xcoords, unsigned long xLen,
                                     const double* yvalues, unsigned long yLen);

    ScanExtremum1D findScanMaximum1D(const double* xcoords, unsigned long xLen,
                                     const double* yvalues, unsigned long yLen);
    //@}
}

#endif // NPSTAT_SCANEXTREMUM1D_HH_
