#ifndef NPSTAT_FCNORCONST_HH_
#define NPSTAT_FCNORCONST_HH_

/*!
// \file fcnOrConst.hh
//
// \brief Templated utilities for use in various density estimation codes, etc
//
// Author: I. Volobouev
//
// June 2022
*/

#include "geners/IOIsClassType.hh"
#include "npstat/nm/SimpleFunctors.hh"

namespace npstat {
    namespace Private {
        template<class Fcn, bool isCallable=true>
        struct FcnOrConstHelper
        {
            inline static double call(const Fcn& fcn, const double x)
                {return fcn(x);}
        };

        template<class Fcn>
        struct FcnOrConstHelper<Fcn, false>
        {
            inline static double call(const Fcn& fcn, const double /* x */)
                {return fcn;}
        };
    }

    template<class Fcn>
    inline double fcnOrConst(const Fcn& functor, const double x)
    {
        return Private::FcnOrConstHelper<Fcn, gs::IOIsClassType<Fcn>::value>::call(functor, x);
    }

    template<class Fcn>
    class MultFunctor
    {
    public:
        inline MultFunctor(const Fcn& fcn, const double f)
            : functor_(fcn), factor_(f) {}

        inline double operator()(const double& x) const
            {return factor_*fcnOrConst(functor_, x);}

    private:
        Fcn functor_;
        double factor_;
    };

    template<class DensityFcn, class Weight>
    class DensitySquaredTimesWeight : public Functor1<double, double>
    {
    public:
        inline DensitySquaredTimesWeight(const DensityFcn& df, const Weight& wf)
            : df_(df), wf_(wf) {}

        inline virtual ~DensitySquaredTimesWeight() {}

        inline double operator()(const double& x) const
        {
            const double d = df_(x);
            const double w = fcnOrConst(wf_, x);
            return d*d*w;
        }

    private:
        DensityFcn df_;
        Weight wf_;
    };

    template<class DensityFcn, class Weight>
    inline DensitySquaredTimesWeight<DensityFcn, Weight>
    make_DensitySquaredTimesWeight(const DensityFcn& df, const Weight& wf)
    {
        return DensitySquaredTimesWeight<DensityFcn, Weight>(df, wf);
    }
}

#endif // NPSTAT_FCNORCONST_HH_
