#ifndef NPSTAT_ASSIGNRESAMPLINGWEIGHTS_HH_
#define NPSTAT_ASSIGNRESAMPLINGWEIGHTS_HH_

/*!
// \file assignResamplingWeights.hh
//
// \brief Resampling with replacement which assigns weights to point locations
//
// Author: I. Volobouev
//
// July 2022
*/

#include <vector>

#include "npstat/rng/AbsRandomGenerator.hh"

namespace npstat {
    /**
    // Imagine that you have a sample of size "nPoints".
    // If you would resample it with replacement in the
    // usuall manner, some points would be chosen a few
    // times, and some not at all. On output of this function,
    // the vector "weightsToFill" will be filled with counts
    // how many times each point of the sample is chosen.
    // Naturally, the size of the "weightsToFill" vector
    // will be set to "nPoints".
    //
    // This is an alternative representation of the resampling
    // procedure which can be useful in certain scenarios.
    */
    void assignResamplingWeights(AbsRandomGenerator& gen,
                                 unsigned long nPoints,
                                 unsigned long sampleSizeWanted,
                                 std::vector<double>* weightsToFill);

#ifdef SWIG
    inline std::vector<double> assignResamplingWeights2(
        AbsRandomGenerator& gen, const unsigned long nPoints,
        const unsigned long sampleSizeWanted)
    {
        std::vector<double> to;
        assignResamplingWeights(gen, nPoints, sampleSizeWanted, &to);
        return to;
    }
#endif
}

#endif // NPSTAT_ASSIGNRESAMPLINGWEIGHTS_HH_
