#include "npstat/nm/SpecialFunctions.hh"
#include "npstat/stat/EllipticalDistributions.hh"

namespace npstat {
    void EllipticalKotz::calcNorm()
    {
        const double n = dim_;
        const double tmp = (2*N_+n-2)/(2*s_);
        norm_ = s_*Gamma(n/2)/pow(M_PI,n/2)/Gamma(tmp)*pow(r_,tmp);
    }

    double EllipticalKotz::density(const double* location1, const unsigned dim1) const
    {
        const double chisq = EllipticalDistribution::chiSquare(location1, dim1);
        double pow0 = 1.0;
        if (N_ != 1U)
            pow0 = pow(chisq, static_cast<int>(N_)-1);
        double pow1 = chisq;
        if (s_ != 1.0)
            pow1 = pow(chisq, s_);
        return pow0*exp(-r_*pow1)*norm_/det_;
    }

    bool EllipticalKotz::write(std::ostream& os) const
    {
        gs::write_pod(os, N_);
        gs::write_pod(os, r_);
        gs::write_pod(os, s_);
        return EllipticalDistribution::classId().write(os) &&
            EllipticalDistribution::write(os);
    }

    EllipticalKotz* EllipticalKotz::read(const gs::ClassId& id, std::istream& in)
    {
        static const gs::ClassId current(
            gs::ClassId::makeId<EllipticalKotz>());
        current.ensureSameId(id);

        unsigned N;
        double r, s;
        gs::read_pod(in, &N);
        gs::read_pod(in, &r);
        gs::read_pod(in, &s);
        CPP11_auto_ptr<EllipticalDistribution> pb =
            gs::read_obj<EllipticalDistribution>(in);
        return new EllipticalKotz(*pb, N, r, s);
    }
}
