#ifndef NPSTAT_LORPE1DCVRESULT_HH_
#define NPSTAT_LORPE1DCVRESULT_HH_

/*!
// \file LOrPE1DCVResult.hh
//
// \brief An object representing the result of the 1-d LOrPE
//        cross-validation procedure
//
// Author: I. Volobouev
//
// August 2022
*/

#include <cassert>

namespace npstat {
    class LOrPE1DCVResult
    {
    public:
        // Default constructor creates an invalid result
        inline LOrPE1DCVResult()
            : filterDegree_(-1.0), bwFactor_(0.0), cvFunction_(0.0),
              onDegreeBoundary_(true), onBwBoundary_(true) {}

        // This constructor must create a valid result
        inline LOrPE1DCVResult(const double i_filterDegree,
                               const double i_bwFactor,
                               const double i_cvFunction,
                               const bool i_onDegreeBoundary,
                               const bool i_onBandwidthBoundary)
            : filterDegree_(i_filterDegree),
              bwFactor_(i_bwFactor),
              cvFunction_(i_cvFunction),
              onDegreeBoundary_(i_onDegreeBoundary),
              onBwBoundary_(i_onBandwidthBoundary)
        {
            assert(this->isValid());
        }

        inline void setDegreeBoundaryFlag(const bool b) {onDegreeBoundary_ = b;}

        inline double filterDegree() const {return filterDegree_;}
        inline double bwFactor() const {return bwFactor_;}
        inline double cvFunction() const {return cvFunction_;}
        inline bool isOnDegreeBoundary() const {return onDegreeBoundary_;}
        inline bool isOnBandwidthBoundary() const {return onBwBoundary_;}

        inline bool isValid() const {return filterDegree_ >= 0.0 && bwFactor_ > 0.0;}

    private:
        double filterDegree_;
        double bwFactor_;
        double cvFunction_;
        bool onDegreeBoundary_;
        bool onBwBoundary_;
    };
}

#endif // NPSTAT_LORPE1DCVRESULT_HH_
