#ifndef NPSTAT_MATRIXFILTER1DBUILDER_HH_
#define NPSTAT_MATRIXFILTER1DBUILDER_HH_

/*!
// \file MatrixFilter1DBuilder.hh
//
// \brief Builder of filters in one dimension using filtering matrices
//        prepared by some other code
//
// The filters are intended for use with the "LocalPolyFilter1D" class.
//
// Author: I. Volobouev
//
// February 2020
*/

#include "npstat/stat/AbsFilter1DBuilder.hh"
#include "npstat/nm/Matrix.hh"

namespace npstat {
    class MatrixFilter1DBuilder : public AbsFilter1DBuilder
    {
    public:
        /** Each row of matrix "m" is a filter */
        inline explicit MatrixFilter1DBuilder(const Matrix<double>& m) : m_(m) {}

        inline virtual ~MatrixFilter1DBuilder() {}

        /** This method is pure virtual in the base, so it must be implemented */
        inline unsigned centralWeightLength() const {return m_.nColumns();}

        /** Internal filters can all be different */
        inline bool keepAllFilters() const {return true;}

        /** The "taper" and "lenTaper" arguments will be ignored */
        virtual PolyFilter1D* makeFilter(const double* taper, unsigned lenTaper,
                                       unsigned binnum, unsigned datalen) const;
    private:
        Matrix<double> m_;
    };
}

#endif // NPSTAT_MATRIXFILTER1DBUILDER_HH_
