#ifndef NPSTAT_POWERTRANSFORM1D_HH_
#define NPSTAT_POWERTRANSFORM1D_HH_

/*!
// \file PowerTransform1D.hh
//
// \brief Transform y = a x^p (a > 0). Obviously, works only for
//        non-negative x. x must be strictly positive if p is negative.
//
// Author: I. Volobouev
//
// September 2022
*/

#include "npstat/stat/AbsDistributionTransform1D.hh"

namespace npstat {
    class PowerTransform1D : public AbsDistributionTransform1D
    {
    public:
        PowerTransform1D(double a, double p);
        inline virtual ~PowerTransform1D() {}

        inline virtual PowerTransform1D* clone() const
            {return new PowerTransform1D(*this);}

        double transformForward(double x, double* dydx) const;
        double transformBack(double y) const;
        inline bool isIncreasing() const {return params_[1] > 0.0;}

        //@{
        /** Prototype needed for I/O */
        inline virtual gs::ClassId classId() const {return gs::ClassId(*this);}
        virtual bool write(std::ostream&) const;
        //@}

        static inline const char* classname()
            {return "npstat::PowerTransform1D";}
        static inline unsigned version() {return 1;}
        static PowerTransform1D* read(const gs::ClassId& id, std::istream& is);

    protected:
        virtual bool isEqual(const AbsDistributionTransform1D&) const;

    private:
        void validateScale(double a);
        void validatePower(double p);

        inline void setParameterChecked(const unsigned which, const double value)
        {
            if (which)
                validatePower(value);
            else
                validateScale(value);
            params_[which] = value;
        }
        inline void setAllParametersChecked(const double* p)
        {
            validateScale(p[0]);
            validatePower(p[1]);
            params_[0] = p[0];
            params_[1] = p[1];
        }
        inline double getParameterChecked(const unsigned which) const
            {return params_[which];}

        double params_[2];
    };
}

#endif // NPSTAT_POWERTRANSFORM1D_HH_
