#ifndef NPSTAT_SMOOTHGOFTEST1D_HH_
#define NPSTAT_SMOOTHGOFTEST1D_HH_

/*!
// \file SmoothGOFTest1D.hh
//
// \brief Smooth test for goodness-of-fit
//
// Author: I. Volobouev
//
// November 2020
*/

#include <memory>

#include "npstat/nm/AbsClassicalOrthoPoly1D.hh"

#include "npstat/stat/AbsUnbinnedGOFTest1D.hh"

namespace npstat {
    class SmoothGOFTest1D : public AbsUnbinnedGOFTest1D
    {
    public:
        // The mask, if provided, should contain "maxdeg" elements.
        // Puts some number in the mask in order to discard the
        // corresponding eigenvector and put 0 there to keep it.
        // The degree numbering in the mask argument starts from 1.
        SmoothGOFTest1D(const AbsDistribution1D& distro,
                        const AbsClassicalOrthoPoly1D& poly,
                        unsigned maxdeg,
                        const unsigned char* mask=0, unsigned lenMask=0);

        inline virtual ~SmoothGOFTest1D() {}

        virtual std::string shortName() const;

        inline unsigned maxDegree() const {return maxdeg_;}
        inline unsigned numDeviations() const {return nUnmasked_;}
        inline bool isDegreeUsed(const unsigned i) const
            {return !i || i > maxdeg_ ? false : !mask_[i];}

        /**
        // Get the value of the test statistic with all
        // terms weighted equally
        */
        inline virtual double testStatistic(
            const double* data, const unsigned long sz, bool) const
            {return testStat(data, sz);}

        inline virtual double testStatistic(
            const float* data, const unsigned long sz, bool) const
            {return testStat(data, sz);}

        inline virtual bool hasAnalyticPValue() const {return true;}

        virtual double analyticPValue(double stat, unsigned long sz) const;

        virtual double inverseExceedance(double pvalue, unsigned long sz,
                                         double smin, double smax) const;

        /**
        // Get individual normalized deviations from which
        // the test statistic is constructed. Only unmasked
        // deviations are returned. The return value of this
        // function is the number of such deviations.
        */
        template<typename Numeric>
        unsigned normalizedDeviations(
            const Numeric* data, unsigned long lenData,
            double* deviations, unsigned lenDeviations) const;

    private:
        template<typename Numeric>
        double testStat(const Numeric* data, unsigned long lenData) const;

        std::shared_ptr<const AbsClassicalOrthoPoly1D> poly_;
        std::vector<double> aveBuf_;
        std::vector<double> statBuf_;
        std::vector<unsigned char> mask_;
        unsigned maxdeg_;
        unsigned nUnmasked_;
    };
}

#include "npstat/stat/SmoothGOFTest1D.icc"

#endif // NPSTAT_SMOOTHGOFTEST1D_HH_
