#ifndef NPSTAT_BIVARIATECHISQUARE_HH_
#define NPSTAT_BIVARIATECHISQUARE_HH_

/*!
// \file bivariateChiSquare.hh
//
// \brief Simple expression for the chi-square in the bivariate case
//
// Author: I. Volobouev
//
// August 2022
*/

/**
// Imagine the bivariate normal density. mux and muy are shifts (means).
// sx and sy are the standard deviations (must be positive).
// rho is the correlation coefficient (must have |rho| < 1.0).
// The function returns the chi-square between the bivariate mean
// and the point (x, y). The bivariate normal density at (x, y)
// is proportional to exp(-chi_square/2).
*/
double bivariateChiSquare(double mux, double muy, double sx, double sy,
                          double rho, double x, double y);

#endif // NPSTAT_BIVARIATECHISQUARE_HH_
