#ifndef NPSTAT_CUMULANTCONVERSION_HH_
#define NPSTAT_CUMULANTCONVERSION_HH_

/*!
// \file cumulantConversion.hh
//
// \brief Conversions between cumulants and central moments
//
// Author: I. Volobouev
//
// July 2019
*/

namespace npstat {
    /**
    // Arrays "cumulants" and "centralMoments" should have at least
    // maxOrder+1 elements, with cumulants[2] set to the variance.
    // On exit, centralMoments[0] will be set to 1, and centralMoments[1]
    // to cumulants[1].
    //
    // Currently, maxOrder can not exceed 20.
    */
    template<typename Real>
    void convertCumulantsToCentralMoments(const Real* cumulants,
                                          unsigned maxOrder,
                                          Real* centralMoments);

    /**
    // Arrays "cumulants" and "centralMoments" should have at least
    // maxOrder+1 elements, with centralMoments[2] set to the variance.
    // On exit, cumulants[0] will be set to 0, and cumulants[1]
    // to centralMoments[1].
    //
    // Currently, maxOrder can not exceed 20.
    */
    template<typename Real>
    void convertCentralMomentsToCumulants(const Real* centralMoments,
                                          unsigned maxOrder,
                                          Real* cumulants);
}

#include "npstat/stat/cumulantConversion.icc"

#endif // NPSTAT_CUMULANTCONVERSION_HH_
