#ifndef NPSTAT_SCANNEDKSDISTANCE_HH_
#define NPSTAT_SCANNEDKSDISTANCE_HH_

/*!
// \file scannedKSDistance.hh
//
// \brief Calculate KS distance between two 1-d distributions by scanning
//        the difference between cdf values
//
// Author: I. Volobouev
//
// November 2019
*/

#include "npstat/stat/AbsDistribution1D.hh"

namespace npstat {
    /**
    // The x values at which the cumulative density functions
    // are compared are determined by spliting the [0, 1] interval
    // into "nScanPoints" subintervals (bins) and then calculating
    // the quantile function of the reference distribution at the
    // center of each subinterval.
    */
    double scannedKSDistance(const AbsDistribution1D& reference,
                             const AbsDistribution1D& compared,
                             unsigned nScanPoints);
}

#endif // NPSTAT_SCANNEDKSDISTANCE_HH_
