#ifndef NPSTAT_PYFUNCTOR1_HH_
#define NPSTAT_PYFUNCTOR1_HH_

#include <cassert>

#include "npstat/nm/SimpleFunctors.hh"
#include "npstat/wrap/pyWrapUtils.hh"

namespace npstat {
    class PyFunctor1 : public Functor1<double,double>
    {
    public:
        inline explicit PyFunctor1(PyObject* callable)
            : pFunc_(0)
        {
            validatePyCallable(callable);
            pFunc_ = callable;
            Py_INCREF(pFunc_);
        }

        inline PyFunctor1(const PyFunctor1& r)
            : pFunc_(r.pFunc_)
        {
            Py_INCREF(pFunc_);
        }

        inline PyFunctor1& operator=(const PyFunctor1& r)
        {
            if (this != &r)
            {
                Py_XDECREF(pFunc_);
                pFunc_ = r.pFunc_;
                Py_INCREF(pFunc_);
            }
            return *this;
        }

        inline virtual ~PyFunctor1() {Py_XDECREF(pFunc_);}

        inline double operator()(const double& x_in) const
        {
            // Build the arguments for the callable
            PyObject *pArgs = PyTuple_New(1);
            DecrefHandle h_pArgs(pArgs);
            PyObject* x = PyFloat_FromDouble(x_in);
            assert(x);
            PyTuple_SetItem(pArgs, 0, x);

            // Evaluate the callable
            PyObject* pValue = PyObject_CallObject(pFunc_, pArgs);
            if (!pValue) throw std::runtime_error(
                "In npstat::PyFunctor1::operator(): object call failed");
            DecrefHandle h_pValue(pValue);

            return getDoubleFromPyObj(pValue);
        }

    private:
        PyObject* pFunc_;
    };
}

#endif // NPSTAT_PYFUNCTOR1_HH_
