#ifndef NPSTAT_HISTOBINCOORDS_HH_
#define NPSTAT_HISTOBINCOORDS_HH_

#include "npstat/wrap/NumpyTypecode.hh"

namespace npstat {
    template <class Axis>
    inline PyObject* histoBinCenters(const Axis& axis,
                                     const bool reverseOrder=false)
    {
        const int typenum = NumpyTypecode<double>::code;

        const int nBins = axis.nBins();
        npy_intp sh = nBins;
        PyObject* xarr = PyArray_SimpleNew(1, &sh, typenum);
        if (xarr)
        {
            double* x = (double*)PyArray_DATA((PyArrayObject*)xarr);
            if (reverseOrder)
                for (int i=0; i<nBins; ++i)
                    x[i] = axis.binCenter(nBins-i-1);
            else
                for (int i=0; i<nBins; ++i)
                    x[i] = axis.binCenter(i);
        }
        return xarr;
    }

    template <class Axis>
    inline PyObject* histoBinBoundaries(const Axis& axis,
                                        const bool reverseOrder=false)
    {
        const int typenum = NumpyTypecode<double>::code;

        const int nBins = axis.nBins();
        npy_intp sh = nBins + 1;
        PyObject* xarr = PyArray_SimpleNew(1, &sh, typenum);
        if (xarr)
        {
            double* x = (double*)PyArray_DATA((PyArrayObject*)xarr);
            if (reverseOrder)
            {
                x[0] = axis.max();
                for (int i=1; i<=nBins; ++i)
                    x[i] = axis.leftBinEdge(nBins-i);
            }
            else
            {
                for (int i=0; i<nBins; ++i)
                    x[i] = axis.leftBinEdge(i);
                x[nBins] = axis.max();
            }
        }
        return xarr;
    }
}

#endif // NPSTAT_HISTOBINCOORDS_HH_
