#ifndef NPSTAT_LORPESMOOTH1DWRAP_HH_
#define NPSTAT_LORPESMOOTH1DWRAP_HH_

#include <stdexcept>

#include "npstat/wrap/NumpyTypecode.hh"
#include "npstat/stat/lorpeSmooth1D.hh"

namespace npstat {
    template <class Histo>
    PyObject* weightedLorpeSmooth1DWrap(
        const Histo& histo, double effectiveSampleSize,
        int m, double maxDegree, const BoundaryHandling& bm,
        double bandwidthFactor=1.0)
    {
        const int typenum = NumpyTypecode<double>::code;

        if (histo.dim() != 1U) throw std::invalid_argument(
            "In npstat::weightedLorpeSmooth1DWrap: "
            "input histogram must be one-dimensional");
        const unsigned long nBins = histo.nBins();
        npy_intp sh = nBins;
        PyObject* array = PyArray_SimpleNew(1, &sh, typenum);
        if (!array) throw std::runtime_error(
            "In npstat::weightedLorpeSmooth1DWrap: "
            "failed to allocate the result array");
        double bw = 0.0;
        try
        {
            bw = weightedLorpeSmooth1D(
                histo, effectiveSampleSize, m, maxDegree,
                (double*)array_data(array), nBins, bm, bandwidthFactor);
        }
        catch (const std::exception& e)
        {
            Py_DECREF(array);
            throw e;
        }
        return Py_BuildValue("(Od)", array, bw);
    }

    template <class Histo>
    PyObject* lorpeSmooth1DWrap(const Histo& histo,
                                int m, double maxDegree, 
                                const BoundaryHandling& bm,
                                double bandwidthFactor=1.0)
    {
        const double n = histo.nFillsInRange();
        return weightedLorpeSmooth1DWrap(histo, n, m, maxDegree,
                                         bm, bandwidthFactor);
    }
}

#endif // NPSTAT_LORPESMOOTH1DWRAP_HH_
