#ifndef NPSTAT_SAMPLEDISTRO1DWITHWEIGHTWRAP_HH_
#define NPSTAT_SAMPLEDISTRO1DWITHWEIGHTWRAP_HH_

#include <cassert>

#include "npstat/stat/sampleDistro1DWithWeight.hh"

namespace npstat {
    template <class AcceptanceFunction>
    PyObject* sampleDistro1DWithWeightWrap(
        const AbsDistribution1D& distro,
        const AcceptanceFunction& fcn,
        AbsRandomGenerator& rng,
        const double sampleSize,
        const SampleSizeInterpretation whichSize)
    {
        std::vector<std::pair<double,double> > sample;
        const npstat::Triple<double,double,double>& triple =
            sampleDistro1DWithWeight(distro, fcn, rng, sampleSize,
                                     whichSize, &sample);
        const unsigned long len = sample.size();
        PyObject* lst = PyList_New(len);
        assert(lst);
        for (unsigned long i=0; i<len; ++i)
        {
            const int status = PyList_SetItem(
                lst, i, Py_BuildValue("(dd)", sample[i].first, sample[i].second));
            assert(!status);
        }
        PyObject* status = Py_BuildValue("(ddd)", triple.first,
                                         triple.second, triple.third);
        assert(status);
        return Py_BuildValue("(OO)", lst, status);
    }
}

#endif // NPSTAT_SAMPLEDISTRO1DWITHWEIGHTWRAP_HH_
