#ifndef NPSTAT_SCANDENSITY1D_HH_
#define NPSTAT_SCANDENSITY1D_HH_

#include "npstat/stat/AbsDistribution1D.hh"
#include "npstat/wrap/NumpyTypecode.hh"

namespace npstat {
    inline PyObject* scanDensity1D(
        const AbsDistribution1D& distro,
        const double xmin, const double xmax, const unsigned npoints)
    {
        const int typenum = NumpyTypecode<double>::code;
        npy_intp sh = npoints;
        PyObject* xarr  = PyArray_SimpleNew(1, &sh, typenum);
        PyObject* yarr  = PyArray_SimpleNew(1, &sh, typenum);
        PyObject* result = NULL;

        if (xarr == NULL || yarr == NULL)
            goto fail;
        else if (npoints)
        {
            double* xv = (double*)array_data(xarr);
            double* yv = (double*)array_data(yarr);
            double step = xmax - xmin;
            if (npoints > 1U)
                step /= (npoints - 1U);
            for (unsigned i=0; i<npoints; ++i)
            {
                double x = xmin + i*step;
                if (i && i+1U == npoints)
                    x = xmax;
                xv[i] = x;
                yv[i] = distro.density(x);
            }

            result = Py_BuildValue("(OO)", xarr, yarr);
            if (result == NULL)
                goto fail;
        }

        return result;

    fail:
        Py_XDECREF(xarr);
        Py_XDECREF(yarr);
        return result;
    }

    inline PyObject* scanDensity1D(
        const AbsDistribution1D& distro,
        const double* coords, const unsigned npoints)
    {
        const int typenum = NumpyTypecode<double>::code;
        npy_intp sh = npoints;
        PyObject* yarr  = PyArray_SimpleNew(1, &sh, typenum);

        if (yarr == NULL)
            goto fail;
        else if (npoints)
        {
            assert(coords);
            double* yv = (double*)array_data(yarr);
            for (unsigned i=0; i<npoints; ++i)
                yv[i] = distro.density(coords[i]);
        }

        return yarr;

    fail:
        Py_XDECREF(yarr);
        return 0;
    }
}

#endif // NPSTAT_SCANDENSITY1D_HH_
