#ifndef NPSTAT_SCANDENSITYND_HH_
#define NPSTAT_SCANDENSITYND_HH_

#include <cassert>
#include <stdexcept>

#include "npstat/stat/AbsDistributionND.hh"
#include "npstat/nm/BoxNDScanner.hh"
#include "npstat/wrap/NumpyTypecode.hh"

namespace npstat {
    inline PyObject* scanDensityND(
        const AbsDistributionND& distro,
        const BoxND<double>& boundingBox,
        const std::vector<unsigned>& shape,
        const bool scanBoxBoundaries = false)
    {
        const unsigned dim = distro.dim();
        if (!dim || dim != boundingBox.dim() || dim != shape.size())
            throw std::invalid_argument(
                "In npstat::scanDensityND: "
                "incompatible argument dimensionalities");
        assert(dim <= CHAR_BIT*sizeof(unsigned long));

        BoxND<double> scanBox(boundingBox);
        if (scanBoxBoundaries)
            for (unsigned i=0; i<dim; ++i)
            {
                if (shape[i] < 2U) throw std::invalid_argument(
                    "In npstat::scanDensityND: must have "
                    "at least two scan points in each dimension");
                const double hover2 = 0.5*(scanBox[i].max() - scanBox[i].min())/(shape[i] - 1U);
                scanBox[i].setBounds(scanBox[i].min() - hover2, scanBox[i].max() + hover2);
            }

        npy_intp sh[CHAR_BIT*sizeof(unsigned long)];
        for (unsigned i=0; i<dim; ++i)
            sh[i] = shape[i];
        const int typenum = NumpyTypecode<double>::code;
        PyObject* arr = PyArray_SimpleNew(dim, &sh[0], typenum);
        if (arr)
        {
            double* data = (double*)PyArray_DATA((PyArrayObject*)arr);
            double coords[CHAR_BIT*sizeof(unsigned long)];
            for (BoxNDScanner<double> scan(scanBox, shape);
                 scan.isValid(); ++scan)
            {
                scan.getCoords(&coords[0], dim);
                data[scan.state()] = distro.density(&coords[0], dim);
            }
        }
        return arr;
    }
}

#endif // NPSTAT_SCANDENSITYND_HH_
